var path = require('path')
var fs = require('fs')
var childProcess = require('child_process')
var extend = require('node.extend.without.arrays')
var Promise = require('bluebird')

// Path to html_to_excel binary
// (installed by npm install html-to-excel-render)
var exePath = "html_to_excel_renderer"

var HtmlToExcel = function (reporter, definition) {
  this.reporter = reporter
  this.definition = definition
  this.execOptions = definition.options.execOptions || {}
  this.execOptions.env = extend(true, {}, process.env, this.execOptions.env)

  reporter.extensionsManager.recipes.push({
    name: 'html_to_excel',
    execute: HtmlToExcel.prototype.execute.bind(this)
  })
}

function createParams (reporter, request, paths) {
  var params = []

  params.push(paths.template)

  const outputPath = path.join(
    path.dirname(paths.template),
    `${path.basename(paths.template, '.html')}.xlsx`
  )

  params.push(paths.data_file)
  params.push(outputPath)
  params.push(1000)
  return params
}


HtmlToExcel.prototype.execute = function (request, response) {
  var self = this
  var reporter = self.reporter
  var options = {}
  var paths = {}

  return reporter.writeTempFile((uuid) => `${uuid}.json`, JSON.stringify(request.data))
  .then(function (result) {
  	paths.data_file = result.pathToFile
  	return reporter.writeTempFile((uuid) => `${uuid}.html`, response.content.toString())
	  .then(function (result) {
	    paths.template = result.pathToFile
	    return self.render_xlsx(createParams(self.reporter, request, paths), request)
			    	.then(function (buf) {
			      		response.meta.contentType = 'application/xlsx'
			      		response.meta.fileExtension = 'xlsx'
			      		response.content = buf
			    	})    
		  })

  }) 
}

HtmlToExcel.prototype.render_xlsx = function (parameters, request) {
  var reporter = this.reporter
  var self = this
  var timeout = reporter.getAvailableRenderTimeout(request, self.execOptions.timeout)
  var execOptions = Object.assign({}, self.execOptions)

  if (timeout != null) {
    execOptions.timeout = timeout
    execOptions.killSignal = 'SIGTERM'
  }


  return new Promise(function (resolve, reject) {
    childProcess.execFile(exePath, parameters, execOptions, function (err, stderr, stdout) {
      reporter.logger.debug((err || '') + (stderr || '') + (stdout || ''), request)

      if (err) {
        if (err.killed && err.signal === 'SIGTERM' && execOptions.timeout != null) {
          return reject(new Error('Timeout Error: html-to-excel generation not completed after ' + execOptions.timeout + 'ms'))
        }

        return reject(err)
      }

      fs.readFile(parameters[2], function (err, buf) {
        if (err) {
          return reject(err)
        }

        resolve(buf)
      })
    })
  })
}

module.exports = function (reporter, definition) {
  reporter.html_to_excel = new HtmlToExcel(reporter, definition)
}
